#include "configfile.h"
#include "Config.h"
#include "avm_output.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h> // atexit
#include <stdio.h>
#include <pwd.h>

AVM_BEGIN_NAMESPACE

/* avoiding using static objects - using just pointers */

#define DEFAULT_DIR ".avm"
#define DEFAULT_NAME "default"

static avm::string sConfigDir(DEFAULT_DIR);
static avm::string sConfigName(DEFAULT_NAME);
static ConfigFile* config = 0;

// called at exit
static void destroy_config()
{
    delete config;
    config = 0;
}

static ConfigFile* get_config()
{
    if (!config)
    {
	const char* home = getenv("HOME");
	if (home == 0)
	{
	    const struct passwd* pwent = getpwuid(getuid());
	    home = pwent->pw_dir;
	}
	avm::string s(home);
	s += "/";
	s += sConfigDir;

        struct stat st;
	if (stat(s.c_str(), &st))
	{
	    AVM_WRITE("Registry", "creating dir: %s\n", s.c_str());
            mkdir(s.c_str(), 0755);
	}

	s += "/";
	s += sConfigName;

	config = new ConfigFile(s.c_str());
	atexit(destroy_config);
    }

    return config;
}

void* RegInit(const char* regname, const char* dirname)
{
    destroy_config();
    sConfigDir = dirname ? : DEFAULT_DIR;
    sConfigName = regname ? : DEFAULT_NAME;

    return get_config();
}

void RegSave()
{
    get_config()->Save();
}

int RegReadInt(const char* appname, const char* valname, int def_value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
    {
	if (e->type == ConfigEntry::Int)
	    return e->i;
	if (e->type == ConfigEntry::Binary
	    && (sscanf(e->value.c_str(), "%d", &e->i) == 1))
	{
	    e->type = ConfigEntry::Int;
            return e->i;
	}
    }
    RegWriteInt(appname, valname, def_value);
    return def_value;
}

int RegWriteInt(const char* appname, const char* valname, int value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
    {
	e->SetInt(value);
	get_config()->dirty = true;
    }
    else
	get_config()->push_back(ConfigEntry(appname, valname, value));
    return 0;
}

float RegReadFloat(const char* appname, const char* valname, float def_value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
    {
	if (e->type != ConfigEntry::Float)
	{
	    if (e->type != ConfigEntry::Binary
		|| (sscanf(e->value.c_str(), "%f", &e->f) != 1))
		return -1;
            e->type = ConfigEntry::Float;
	}
	return e->f;
    }
    RegWriteFloat(appname, valname, def_value);
    return def_value;
}

int RegWriteFloat(const char* appname, const char* valname, float value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
    {
	e->SetFloat(value);
	get_config()->dirty = true;
    }
    else
	get_config()->push_back(ConfigEntry(appname, valname, value));
    return 0;
}

const char* RegReadString(const char* appname, const char* valname, const char* def_value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
	return (e->type == ConfigEntry::Binary) ? e->value.c_str() : "";

    RegWriteString(appname, valname, def_value);
    return def_value;
}

int RegWriteString(const char* appname, const char* valname, const char* value)
{
    if (ConfigEntry* e = get_config()->Find(appname, valname))
    {
	e->SetString(value);
	get_config()->dirty = true;
    }
    else
	get_config()->push_back(ConfigEntry(appname, valname, value));
    return 0;
}

AVM_END_NAMESPACE;
